import { VerificationError, apply_number, apply_string, apply_boolean, apply_void } from "./ts_base";
import AddValueResponse, { apply_AddValueResponse } from "./AddValueResponse";
import AddValueRequest, { apply_AddValueRequest } from "./AddValueRequest";
export type {
   AddValueResponse,
   AddValueRequest,
}
import { Service } from "./service_server";

export abstract class TestService<T> extends Service<T> {
   public name = "TestService";
   constructor(){
      super();
      this.functions.add("AddValuesSingleParam")
      this.functions.add("AddValuesMultipleParams")
      this.functions.add("ReturningVoid")
      this.functions.add("OnEvent")
      this.functions.add("ThrowingError")
      this.functions.add("FunctionWithArrayAsParamAndReturn")
   }

   abstract AddValuesSingleParam(request: AddValueRequest, ctx: T): Promise<AddValueResponse>;
   _AddValuesSingleParam(params: any[] | any, ctx: T): Promise<AddValueResponse> {
      let p: any[] = [];
      if(Array.isArray(params)){
         p = params;
         while(p.length < 1)
            p.push(undefined)
      } else {
         p.push(params["request"])
      }
      
      if(p[0] !== null && p[0] !== undefined) {
      apply_AddValueRequest(p[0])
      }
      
      p.push(ctx);
      //@ts-ignore This will cause a typescript error when strict checking, since p is not a tuple
      return this.AddValuesSingleParam.call(this, ...p).then(res => apply_AddValueResponse(res));
   }

   abstract AddValuesMultipleParams(value1: number, value2: number, ctx: T): Promise<number>;
   _AddValuesMultipleParams(params: any[] | any, ctx: T): Promise<number> {
      let p: any[] = [];
      if(Array.isArray(params)){
         p = params;
         while(p.length < 2)
            p.push(undefined)
      } else {
         p.push(params["value1"])
         p.push(params["value2"])
      }
      
      if(p[0] !== null && p[0] !== undefined) {
      apply_number(p[0])
      }
      if(p[1] !== null && p[1] !== undefined) {
      apply_number(p[1])
      }
      
      p.push(ctx);
      //@ts-ignore This will cause a typescript error when strict checking, since p is not a tuple
      return this.AddValuesMultipleParams.call(this, ...p).then(res => apply_number(res));
   }

   abstract ReturningVoid(param1: number, ctx: T): Promise<void>;
   _ReturningVoid(params: any[] | any, ctx: T): Promise<void> {
      let p: any[] = [];
      if(Array.isArray(params)){
         p = params;
         while(p.length < 1)
            p.push(undefined)
      } else {
         p.push(params["param1"])
      }
      
      if(p[0] !== null && p[0] !== undefined) {
      apply_number(p[0])
      }
      
      p.push(ctx);
      //@ts-ignore This will cause a typescript error when strict checking, since p is not a tuple
      return this.ReturningVoid.call(this, ...p).then(res => apply_void(res));
   }

   abstract OnEvent(param1: string, ctx: T): void;
   _OnEvent(params: any[] | any, ctx: T): void {
      let p: any[] = [];
      if(Array.isArray(params)){
         p = params;
         while(p.length < 1)
            p.push(undefined)
      } else {
         p.push(params["param1"])
      }
      
      if(p[0] !== null && p[0] !== undefined) {
      apply_string(p[0])
      }
      
      p.push(ctx);
      //@ts-ignore This will cause a typescript error when strict checking, since p is not a tuple
      return this.OnEvent.call(this, ...p)
   }

   abstract ThrowingError(ctx: T): Promise<void>;
   _ThrowingError(params: any[] | any, ctx: T): Promise<void> {
      let p: any[] = [];
      if(Array.isArray(params)){
         p = params;
         while(p.length < 0)
            p.push(undefined)
      } else {
      }
      
      
      p.push(ctx);
      //@ts-ignore This will cause a typescript error when strict checking, since p is not a tuple
      return this.ThrowingError.call(this, ...p).then(res => apply_void(res));
   }

   abstract FunctionWithArrayAsParamAndReturn(values1: number[], values2: number[], ctx: T): Promise<number[]>;
   _FunctionWithArrayAsParamAndReturn(params: any[] | any, ctx: T): Promise<number[]> {
      let p: any[] = [];
      if(Array.isArray(params)){
         p = params;
         while(p.length < 2)
            p.push(undefined)
      } else {
         p.push(params["values1"])
         p.push(params["values2"])
      }
      
      if(p[0] !== null && p[0] !== undefined) {
      for(const elm of p[0]) {
         apply_number(elm)
      }
      }
      if(p[1] !== null && p[1] !== undefined) {
      for(const elm of p[1]) {
         apply_number(elm)
      }
      }
      
      p.push(ctx);
      //@ts-ignore This will cause a typescript error when strict checking, since p is not a tuple
      return this.FunctionWithArrayAsParamAndReturn.call(this, ...p).then(res => res.map(e => apply_number(e)));
   }

}